MAX_CONTROL_POINTS <- 8
TEAM_UNASSIGNED <- 0
TF_TEAM_BLUE <- Constants.ETFTeam.TF_TEAM_BLUE

function OnPostSpawn() {
	Elevator <- Entities.FindByName(null, "cp_2_plat")

	StartPath <- Entities.FindByName(null, "cap_track1")
	EndPath <- Entities.FindByName(null, "cap_track8")
	StartHeight <- StartPath.GetOrigin().z
	EndHeight <- EndPath.GetOrigin().z
	TotalHeight <- EndHeight - StartHeight

	PointIndex <- 1
	PointTeamIndex <- MAX_CONTROL_POINTS * TF_TEAM_BLUE + PointIndex

	ObjectiveResource <- Entities.FindByClassname(null, "tf_objective_resource")

	NetProps.SetPropBoolArray(ObjectiveResource, "m_bCPCapRateScalesWithPlayers", false, PointIndex)
	NetProps.SetPropBoolArray(ObjectiveResource, "m_bTeamCanCap", true, PointTeamIndex)
	NetProps.SetPropFloatArray(ObjectiveResource, "m_flTeamCapTime", 10000, PointTeamIndex)

	FlagZoneActive <- false
	FlagZone <- Entities.FindByName(null, "cp_2_interrupt_bludetect")
	FlagZone.ValidateScriptScope()
	local flagZoneScope = FlagZone.GetScriptScope()

	flagZoneScope.OnEnterFlagZone <- OnEnterFlagZone
	flagZoneScope.OnLeaveFlagZone <- OnLeaveFlagZone
	flagZoneScope.FlagZoneThink <- FlagZoneThink
	FlagZone.ConnectOutput("OnStartTouchAll", "OnEnterFlagZone")
	FlagZone.ConnectOutput("OnEndTouch", "OnLeaveFlagZone")
	AddThinkToEnt(FlagZone, "FlagZoneThink")

	BlockZoneActive <- false
	BlockZone <- Entities.FindByName(null, "cp_2_interrupt_reddetect")
	BlockZone.ValidateScriptScope()
	local blockZoneScope = BlockZone.GetScriptScope()

	blockZoneScope.OnEnterBlockZone <- OnEnterBlockZone
	blockZoneScope.OnLeaveBlockZone <- OnLeaveBlockZone
	blockZoneScope.OnBlockZoneEmpty <- OnBlockZoneEmpty
	BlockZone.ConnectOutput("OnStartTouch", "OnEnterBlockZone")
	BlockZone.ConnectOutput("OnEndTouch", "OnLeaveBlockZone")
	BlockZone.ConnectOutput("OnEndTouchAll", "OnBlockZoneEmpty")
}

function OnEnterFlagZone[this]() {
	FlagZoneActive <- true
	RecalculateState()

	SetTeardropVisible(true)

	NetProps.SetPropIntArray(ObjectiveResource, "m_iCappingTeam", TF_TEAM_BLUE, PointIndex)
}

function OnLeaveFlagZone[this]() {
	FlagZoneActive <- false
	RecalculateState()

	SetTeardropVisible(false)
}

function FlagZoneThink[this]() {
	local fraction = (EndHeight - Elevator.GetOrigin().z) / TotalHeight
	if (fraction < 0) fraction = 0
	if (fraction > 1) fraction = 1

	if (fraction >= 0.975 && !FlagZoneActive) {
		NetProps.SetPropIntArray(ObjectiveResource, "m_iCappingTeam", TEAM_UNASSIGNED, PointIndex)
	}

	NetProps.SetPropFloatArray(ObjectiveResource, "m_flCapPercentages", fraction, PointIndex)
	NetProps.SetPropFloatArray(ObjectiveResource, "m_flLazyCapPerc", fraction, PointIndex)

	return -1
}

function OnEnterBlockZone[this]() {
	BlockZoneActive <- true
	RecalculateState()

	SetTeardropVisible(true)
}

function OnLeaveBlockZone[this]() {
	SetTeardropVisible(false)
}

function OnBlockZoneEmpty[this]() {
	BlockZoneActive <- false
	RecalculateState()
}

function RecalculateState() {
	local members = (FlagZoneActive && !BlockZoneActive) ? 1 : 0
	local blocked = FlagZoneActive && BlockZoneActive
	NetProps.SetPropIntArray(ObjectiveResource, "m_iNumTeamMembers", members, PointTeamIndex)
	NetProps.SetPropBoolArray(ObjectiveResource, "m_bBlocked", blocked, PointIndex)
}

function SetTeardropVisible(visible) {
	local event = visible ? "controlpoint_starttouch" : "controlpoint_endtouch"
	SendGlobalGameEvent(event, {
		player = activator.entindex()
		area = PointIndex
	})
}
